<div class="">
    <h4><span style="color: green; font-family: sans-serif; font-size: 1.1em"><b>{{ title.title }}</b> <!-- ({{ title.version_current }}) --></span>
  <!-- <span style="float: right; font-size: 15px"><a href="#" onclick="(confirm('Close {{ title.title }}?')) ? (function(){window.open('','_self').close()}()) : (function(){return false;}())">&#x274C; Close App and End Session</a></span> -->
        <span style="float: right; font-size: 15px">
            <a class='btn btn-normal'  href="/instruction" target="_blank" rel="noopener" data-toggle="tooltip" title="Instruction for preparing your
                      device for a scan."><u>Scan Instruction</u></a>
            <a class='btn btn-normal' href="/privacy" target="_blank" rel="noopener" data-toggle="tooltip" title="Review the built-in sharing feature settings of your device with this."><u>Privacy Checkup</u></a>
            <a href="/kill" onclick="window.open('','_self').close();">&#x274C; Close App and End Session</a>
        </span> 
    </h4>

    <p><!-- {{ title.title }} is  -->An <a href="https://github.com/rchatterjee/phone_scanner" target="_blank" rel="noopener">open-source</a> app
        for scanning mobile devices (e.g., Android and iOS devices) for apps that
        can be used to spy on, track, or otherwise monitor a person.<br/>
    </p>
</div>

{% if clients %}

  <div class="col-md-3 border py-2" style="height:80vh;">
        <div align="center">
            <label class="font-weight-bold lead">Which client form would you like to edit?</label>
          <form id="clientnote" method=post>
            <div class="input-group">
              <select class="custom-select" name="clientnote" required>
                <option selected value=""></option>
                {% for client in clients %}
                    <option value="{{ client.id }}" 
                             > {{ client.clientid }} </option>
                        {% endfor %}
              </select>
              <div class="input-group-append">
                <button class="btn btn-primary mx-2">Edit form</button>
              </div>
      </div>
            </form>
   </div>
  </div>
{% else %}

There are no client forms in the database. <a href="/">Submit some</a>, and then come back here!

{% endif %}
