{% macro render_field(field, class_to_use) %}
  <dt>{{ field.label }}</dt>
  <div class="col-auto">
    <dd>{{ field(class_=class_to_use, **kwargs)|safe }}
  </div>
  {% if field.errors %}
    <ul class=errors>
    {% for error in field.errors %}
      <li>{{ error }}</li>
    {% endfor %}
    </ul>
  {% endif %}
  </dd>
{% endmacro %}
