<div class="container">
    <div>
    <a href="#" onclick="close_window('Close these device setup instructions?');return false;">close these instructions</a>
        <div class="row">
            <div class="border p-4">
		    <h3>Documentation of <span style="color: green; font-family: monospace; font-size: 0.3in">{{ title.title }}</span></h3>
                Simple tool to check Android or iOS mobile devices for IPS-spyware.
                Code: <code><a href="https://github.com/stopipv/isdi">available on GitHub</a></code>

                <h4>Prepare your device for scanning</h4>
		You must connect your Android or iOS device with a USB charging cable to the computer that is running
		the {{ title.title }}. In order to allow the scanner to communicate with your device, please follow the
		steps below for the type of device you own.<br><br>

                <h5>iOS (iPhone, iPad)</h5>
		<img src="/webstatic/images/apple.resized.png">
                There is not much preparation required for iOS, just connect the USB cable, on
                the device you should see a prompt asking for the device passcode (sometimes called a "pin" code)
                number. The passcode has to be entered to unlock the device, and trust the
                computer to allow scan the iPhone. 
                <b>Troubleshooting:</b> Sometimes the {{ title.title }} might fail to recognize the
                iOS device. Try opening iTunes on macOS, and see if the device is
                listed. Disconnect the device from the USB cable, reconnect, and try scanning again. 

                <h5>Android Devices</h5>
		<img src="/webstatic/images/android.resiz.png">
                For an Android device, the <code>developer options</code> on the device need to be
		activated. Developer options provide functionality for {{ title.title }} to communicate with 
		the device. The exact steps might vary from device manufacturer and version of
                Android, but roughly the following steps will help you activate developer
                options and USB debugging. 
		
		<br> <br>
		<b>Note about developer options:</b> {{ title.title }} will try to deactivate the developer
                mode at the end of the scan. For some devices this procedure may not work, so if
                    you anticipate any issues with modifying your device's settings, 
		    then please don't turn it on, or scan your device.

                <ol>
                    <li>Go to Settings. Either find the Settings app in the device drawer, or
                        pull down from the top notification bar, on the top right you will see a gear
                        type icon (<b>&#9881;</b>) for settings.</li>
                    <li>Scroll down to find About Phone, search for <code>Build number</code>,
                        tap on the build number 6-8 times to activate the developer mode. For some
                        devices, Build number might be hidden under Software info. (<b>Warning:</b> For
                        most device we can turn it off (see step 4), but there are some devices this
                        cannot be undone. This will not interfere with any of the device’s common
                        functionality, but does not leave the device at the state as it was before the
                        scanning.)</li>
                    <li>Next step is to turning on <code>USB debugging</code>. Go to
                        <code>Settings</code>&rarr;<code>Developer options</code>&rarr;<code>USB debugging</code>.
                        Tap on the toggle switch to turn it on.

			<br></br>
			<img src="/webstatic/images/usb_android.resized.png" style="display: block; margin: 0 auto;"><br>
			<b>Note:</b> You may need to check the device's notification tray to ensure the 
			connection is being used for <code>'File Transfer'</code>. If there is a message from
			the system with 'Charging this device via USB', tap it and select <code>'File Transfer'</code> under 'Use USB For'.
		    </li>
                    <!-- 
                         the developer mode is turned on, connect the device to the laptop using the
                         correct USB cable. In the device a dialogue box will appear with message "Do you
                         trust this computer?" (or something similar).  Saying okay to this will allow
                         the computer the scan the device via the USB cable.  Tap okay.  Press the
                         “Android” button on the left side of the Phone Scanner page in browser. Let the
                         scan finish.  (Troubleshooting: If {{ title.title }} does not find any app, refresh
                         the page once. If still doesn’t work, disconnect the USB cable from the device,
                         reconnect, and refresh the page. Last option, manually scroll over the apps on
                         the device by going to ‘Settings → Apps’, and see if anything looks suspicious.
                         Once the scan is done, the page will look like the following.  There are three
                         main input the researcher has to enter.  Client ID (The researcher has to pick
                         the client ID) The left side notes are any general notes about the client the
                         researchers want to record.  There are fields for taking notes for each App,
                         record any comment form the client about the apps (especially the ones which are
                         flagged) in the Notes input field beside each app.   -->
                        <li><b>Turning off developer options:</b>
                            <!-- Finally, after the interview is over, press the save button to save
                                 the information in the database. To reset the page press the Reset. [Note: this
                                 will also create a database entry, just in case someone accidentally presses
                                 reset.]   -->
                            The scanner will automatically try to reset the developer options
                            once Save or Reset is pressed. Restart the device to get to original
                            state. If you still see the developer options (a.k.a, developer
                            mode) is on, open <code>Settings</code>&rarr;<code>Apps</code>, you
                            will see list of apps installed in your device. Find the app
                            <code>Settings</code> in list of apps, tap on it, and tap clear data
                            (sometimes clear data option is listed inside Storage).  This should
                            clean the developer options. [Warning: For some Android, device this
                            might fail. So please check the settings again. developer options ,
                            but not all.]</li>
                </ol>

            </div>
        </div>
        
        <br/><br/>
        <div class="border p-4"> 
            Send your feedback to <a href="mailto:rahul@cs.cornell.edu">rahul@cs.cornell.edu</a>
        </div>
    </div>
</div>
